import ring
import logging
from .base import APIBase, CurrencyImplBase


class API(APIBase, name='upbit'):

    URL_PREFIX = 'https://crix-api.upbit.com/v1'
    RECENT_URL = f'{URL_PREFIX}/crix/recent'
    HEADERS = {
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
        'Accept-Encoding': 'gzip, deflate, br',
        'Accept-Language': 'en-US,en',
        'Connection': 'keep-alive',
        'DNT': '1',
        'Upgrade-Insecure-Requests': '1',
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.13; rv:57.0) Gecko/20100101 Firefox/57.0'
    }
    FORMAT_CRIX_CODE = 'CRIX.UPBIT.{to_currency}-{from_currency}'.format

    def __ring_key__(self):
        return 'upbit'

    def fetch_recent(self, codes):
        url = self.RECENT_URL
        combined_codes = ','.join(codes)
        params = {'codes': combined_codes}
        response = self.session.get(url, params=params, headers=self.HEADERS)
        data = response.json()
        for row in data:
            self.recent.set(row, row['code'])
        return data

    @ring.func.dict({}, expire=5.0)  # 5 seconds cache
    def recent(self, code):
        full_data = self.fetch_recent([code])
        logging.warning('fetch full:', full_data)
        data = full_data[0]
        return data


class Currency(CurrencyImplBase, api=API):

    def last(self):
        code = self.api.FORMAT_CRIX_CODE(
            to_currency=self.pair[1], from_currency=self.pair[0])
        r = self.api.recent(code)
        return r['tradePrice']



from coinwraps.base import CurrencyPair
import coinwraps.bithumb  # noqa
import coinwraps.coinone  # noqa
import coinwraps.korbit  # noqa
import coinwraps.upbit  # noqa

__all__ = 'CurrencyPair',

bithumb_api = coinwraps.bithumb.API.shared_api
coinone_api = coinwraps.coinone.API.shared_api
korbit_api = coinwraps.korbit.API.shared_api
upbit_api = coinwraps.upbit.API.shared_api



from .base import APIBase, CurrencyImplBase
import ring


class API(APIBase, name='korbit'):

    URL_PREFIX = 'https://api.korbit.co.kr'
    TICKER_URL = f'{URL_PREFIX}/v1/ticker/detailed'

    def __ring_key__(self):
        return 'korbit'

    @ring.func.dict({}, expire=5)
    def ticker(self, currency):
        url = self.TICKER_URL
        response = self.session.get(
            url, params={'currency_pair': f'{currency.lower()}_krw'})
        parsed_data = response.json()
        return parsed_data


class Currency(CurrencyImplBase, api=API):

    def __init__(self, api, pair):
        c1, c2 = pair
        assert c2 == 'KRW'
        super().__init__(api, pair)

    def last(self):
        r = self.api.ticker(self.pair[0].lower())
        return float(r['last'])



from typing import Tuple, Dict
import time
import requests
import logging


class RegistryBase:

    registrations: Dict[str, str]

    def __init__(self):
        self.registrations = {}

    def register(self, key, value):
        self.registrations[key] = value

    def get(self, key):
        return self.registrations[key]


class APIBase:
    registry = RegistryBase()
    name: str

    def __init_subclass__(cls, name, **kwargs):
        cls.name = name
        cls.shared_api = cls()
        cls.registry.register(name, cls.shared_api)

    def __init__(self):
        self.session = requests.Session()
        original_request = self.session.request
        last_time = 0.0

        def safe_request(*args, **kwargs):
            nonlocal last_time
            now = time.time()
            if now - last_time < 1.0:
                sleep_time = 1 - (now - last_time)
                logging.warning(f'sleep {sleep_time} to prevent attack')
                time.sleep(sleep_time)
            last_time = time.time()
            response = original_request(*args, **kwargs)
            return response

        self.session.request = safe_request

    def currency(self, pair):
        return self.currency_impl(self, pair)


class CurrencyImplBase:

    def __init_subclass__(cls, api):
        api.currency_impl = cls

    def __init__(self, api, pair):
        self.api = api
        self.pair = pair


class CurrencyPair:
    pair: Tuple[str, str]

    def __init__(self, pair):
        c1, c2 = [c.upper() for c in pair]
        self.pair = c1, c2

    def exchange(self, name):
        api = APIBase.registry.get(name)
        return api.currency(self.pair)



from __future__ import with_statement

from setuptools import setup


def get_version():
    return '0.0.4'


tests_require = [
    'pytest>=3.0.2', 'pytest-cov', 'pytest-lazy-fixture', 'mock', 'patch',
    'requests',
]

dev_require = tests_require


setup(
    name='coinwraps',
    version=get_version(),
    description='coinwraps',
    long_description='',
    author='Jeong YunWon',
    author_email='coinwraps@youknowone.org',
    url='https://github.com/youknowone/coinwraps',
    packages=(
        'coinwraps',
    ),
    package_data={
        'coinwraps': [],
    },
    install_requires=[
        'attrs',
        'requests',
        'ring==0.6.0',
    ],
    tests_require=tests_require,
    extras_require={
        'tests': tests_require,
        'dev': dev_require,
    },
    classifiers=[
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],
)  # noqa



import yaml


class Config:

    def __init__(self, data):
        self.config = data

    @classmethod
    def from_yaml_file(cls, path):
        data = yaml.load(open(path).read())
        return cls(data)


config = Config.from_yaml_file('config.yaml')


from .base import APIBase, CurrencyImplBase
import ring


class API(APIBase, name='bithumb'):

    URL_PREFIX = 'https://api.bithumb.com'
    TICKER_URL = (URL_PREFIX + '/public/ticker/{currency}').format

    def __ring_key__(self):
        return 'bithumb'

    @ring.func.dict({}, expire=5)
    def ticker(self, currency):
        url = self.TICKER_URL(currency=currency)
        response = self.session.get(url)
        parsed_data = response.json()['data']
        if currency == 'ALL':
            for code, row in parsed_data.items():
                self.ticker.set(row, code)
        return parsed_data


class Currency(CurrencyImplBase, api=API):

    def __init__(self, api, pair):
        c1, c2 = pair
        assert c2 == 'KRW'
        super().__init__(api, pair)

    def last(self):
        data = self.api.ticker(self.pair[0])
        p = (float(data['buy_price']) + float(data['sell_price'])) / 2
        return p
